#!/usr/bin/env python
# coding: UTF-8

import serial
import datetime
import os
from collections import OrderedDict

from appbase import AppBase

class AppTwelite(AppBase):
	# コンストラクタ
	def __init__(self, port=None, baud=115200, tout=0.1 ):
		super( AppTwelite, self ).__init__( port, baud, tout, App='AppTwelite', smode='Ascii')
		self.InitDict()

	# デストラクタ
	def __del__(self):
		self.SerialClose()
		if self.file != None and self.b_openfile :
			self.FileClose()

	# シリアルデータを読み込んで解釈する
	def ReadSensorData(self):
		self.InitDict()
		self.ByteArr = self.SerialRead()
		if self.ByteArr != None:
			self.ReadDict['ArriveTime'] = datetime.datetime.today()
			self.ReadDict['Command'] = self.ByteArr[1]
			self.ReadDict['LogicalID'] = self.ByteArr[0]

			if self.ReadDict['Command'] == 0x81 and len(self.ByteArr) == 23:
				self.ReadDict['EndDeviceSID'] = self.BinList2StrHex(self.ByteArr[5:9])
				self.ReadDict['Identifier'] = self.ByteArr[2]
				self.ReadDict['ProtocolVersion'] = self.ByteArr[3]
				self.ReadDict['LQI'] = self.ByteArr[4]
				self.ReadDict['ReceiverLogicalID'] = self.ByteArr[9]
				self.ReadDict['TimeStamp'] = self.BinList2Int(self.ByteArr[10:12])/64.0
				self.ReadDict['RelayNum'] = self.ByteArr[12]
				self.ReadDict['Power'] = self.BinList2Int(self.ByteArr[13:15])
				self.ReadDict['DI1'] = self.ByteArr[16]&0x01
				self.ReadDict['DI2'] = (self.ByteArr[16]&0x02)>>1
				self.ReadDict['DI3'] = (self.ByteArr[16]&0x04)>>2
				self.ReadDict['DI4'] = (self.ByteArr[16]&0x08)>>3
				self.ReadDict['DI1Mask'] = self.ByteArr[17]&0x01
				self.ReadDict['DI2Mask'] = (self.ByteArr[17]&0x02)>>1
				self.ReadDict['DI3Mask'] = (self.ByteArr[17]&0x04)>>2
				self.ReadDict['DI4Mask'] = (self.ByteArr[17]&0x08)>>3

				__ADC = self.ConvertADC(self.ByteArr[18:23])
				self.ReadDict['AI1'] = __ADC[0]
				self.ReadDict['AI2'] = __ADC[1]
				self.ReadDict['AI3'] = __ADC[2]
				self.ReadDict['AI4'] = __ADC[3]

			return True
		else:
			return False

	def ConvertADC(self, ADCList):
		__ADCDec = list()

		i = 0
		while i < 4:
			__ADC = ADCList[i]<<2
			__ADC += (ADCList[4]>>(2*i))&0x03
			__ADC *= 4
			__ADCDec.append(__ADC)
			i += 1
		return __ADCDec

	# 辞書の中のデータを標準出力する
	def ShowSensorData(self):
		# 辞書のキーを取得する
		__KeyList = self.ReadDict.keys()

		# コンソールをクリアする
		print( '%c[2J%c[H' % (27, 27) )

		# キーごとに出力を変える
		for keys in __KeyList:
			# 出力する文字列を入れる変数の初期化
			__PrintStr = ''
			# 要素を入れる
			__Element = self.ReadDict[keys]

			# 到来時間
			if keys == 'ArriveTime':
				__PrintStr = __Element.strftime('%4Y/%02m/%02d %02H:%02M:%02S.')
				__PrintStr += '%03d' % __Element.microsecond

			elif keys == 'Command':
				__PrintStr = '0x%02X' % __Element

			# SID
			elif keys.find('SID') >= 0:
				# 要素の中身が0x80000000の場合、中継していないのでその旨を出力する
				if __Element == '80000000':
					__PrintStr = 'No Relay'
				# 最上位ビットが不要なため消す
				else:
					__PrintStr = __Element[1:len(self.ReadDict[keys])]

			elif keys.find('ID') >= 0:
				__PrintStr = '0x%02X' % __Element

			# LQI
			elif keys == 'LQI':
				__dbm = (7.0 * __Element - 1970.0) / 20.0
				__PrintStr = str(__Element) + ' (%.02f [dBm])' % __dbm

			# 電源電圧
			elif keys == 'Power':
				__PrintStr = str(__Element) + ' [mV]'

			elif keys == 'AI1':
				__PrintStr = 'AI1 : %4d [mV]' % self.ReadDict['AI1']
				__PrintStr += '  AI2 : %4d [mV]' % self.ReadDict['AI2']
				__PrintStr += '  AI3 : %4d [mV]' % self.ReadDict['AI3']
				__PrintStr += '  AI4 : %4d [mV]' % self.ReadDict['AI4']
				print(__PrintStr)
				continue

			# ADC1ですべて表示しているのでその他のADCは飛ばす
			elif keys.find('AI') >= 0:
				continue

			# DI
			elif keys == 'DI1':
				__PrintStr = 'DI1 : ' + str(self.ReadDict['DI1']) + '/' + str(self.ReadDict['DI1Mask'])
				__PrintStr += '  DI2 : ' + str(self.ReadDict['DI2']) + '/' + str(self.ReadDict['DI2Mask'])
				__PrintStr += '  DI3 : ' + str(self.ReadDict['DI3']) + '/' + str(self.ReadDict['DI3Mask'])
				__PrintStr += '  DI4 : ' + str(self.ReadDict['DI4']) + '/' + str(self.ReadDict['DI4Mask'])
				print(__PrintStr)
				continue

			# DI1ですべて表示しているのでその他のDIは飛ばす
			elif keys.find('DI') >= 0:
				continue

			elif keys == 'TimeStamp':
				__PrintStr = '%.03f' % __Element + ' [s]'

			# 出力用の文字列に中身が入っている場合はそちらを出力する
			if __PrintStr != '':
				print( keys + ' : ' + __PrintStr )
			# 出力用の文字列に何も入っていない場合はそのまま出力する
			else:
				print( keys + ' : ' + str(__Element) )

	def CreateOutputList(self):
		Outlist = list()

		# 辞書のキーを取得する
		__KeyList = self.ReadDict.keys()

		# キーごとに出力を変える
		for keys in __KeyList:
			# 出力する文字列を入れる変数の初期化
			__OutStr = ''
			# 要素を入れる
			__Element = self.ReadDict[keys]

			# 到来時間
			if keys == 'ArriveTime':
				__OutStr = "\t" + __Element.strftime('%4Y/%02m/%02d %02H:%02M:%02S.')
				__OutStr += '%03d' % __Element.microsecond

			# SID
			elif keys.find('SID') >= 0:
				__OutStr = '\t' + __Element[1:len(self.ReadDict[keys])]

			elif keys.find('ID') >= 0 or keys == 'Command' or keys == 'Identifier':
				__OutStr = '\t%02X' % __Element

			else:
				__OutStr = __Element

			Outlist.append(__OutStr)

		return Outlist

# 実装実験用Main関数
if __name__ == '__main__':
	Twelite = AppTwelite(port='/dev/ttyUSB0')

	try:
		while True:
			if Twelite.ReadSensorData():
				Twelite.ShowSensorData()
				Twelite.OutputData( Twelite.CreateOutputList() )

	except KeyboardInterrupt:
		del Twelite
